/**************
 *
 * Filename:    connectmgr.c
 *
 * Purpose:     Connection Manager application
 *
 * Copyright: © 2017 Sierra Wireless Inc., all rights reserved
 *
 **************/
#define __STDC_FORMAT_MACROS
#include <pthread.h>
#include <inttypes.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <time.h>
#include <fcntl.h>
#include <sys/msg.h>
#include <unistd.h>
#include <sys/wait.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <errno.h>

#include "displaysettings.h"
#include "msgid.h"
#include "dms.h"
#include "nas.h"
#include "wds.h"

/****************************************************************
*                       DEFINES
****************************************************************/
#define SUCCESS                  0
#define FAIL                     1
#define ENTER_KEY                0x0A
#define ENTER_KEY_PRESSED        0
#define OPTION_LEN               5
#define IP_ADDRESS_LEN           15
#define IPADDREESS_OCTET_MASK    0x000000FF
#define PDP_IPV4                 0
#define IPv4_FAMILY_PREFERENCE   0x04
#define IPv6_FAMILY_PREFERENCE   0x06
#define IPv4v6_FAMILY_PREFERENCE 0x07
#define MAX_FIELD_SIZE           128

/* APN, User Name and Profile name size should be 3 greater than the actual
 * characters to be read. This will help to ensure that user should not enter
 * more than maximum allowed characters.
 */
#define MAX_APN_SIZE            104
#define MAX_PROFILE_NAME_SIZE   17
#define MAX_USER_NAME_SIZE      129
#define MAX_PROFILES            16
#define PROFILE_TYPE_UMTS       0
#define MIN_PROFILES            1
#define START_DATA_SEESION      1
#define STOP_DATA_SEESION       0
#define TECHNOLOGY_3GPP         1
#define TECHNOLOGY_3GPP2        2

#define CDMA_PROFILE_OFFSET     (100)

/****************************************************************
*                       DATA STRUCTURE
****************************************************************/

enum
{
    eLITE_CONNECT_APP_OK,   
    eLITE_CONNECT_APP_ERR_QMI,
};

/* User options enumeration */
enum eUserOptions{
    eSTART_UMTS_DATA_SESSION = 1,
    eSTART_LTE_DATA_SESSION,
    eSTART_CDMA_DATA_SESSION,
    eSTART_CDMA_RUIM_DATASESSION,
    eSTOP_DATA_SESSION,
    eDISPLAY_ALL_PROFILES,
    eDISPLAY_SINGLE_PROFILE,
    eCREATE_PROFILE,
    eMODIFY_PROFILE_SETTINGS,
    eDELETE_PROFILE,
    eRESET_DEVICE,
    eSET_POWER_MODE,
    eQOS_TESTING
};

/* Profile indexes for profile existing on device */
struct profileIndexesInfo{
    uint8_t profileIndex[MAX_PROFILES];
    uint8_t totalProfilesOnDevice;
};


/* Profile Information structure */
struct profileInformation{
    uint8_t profileType;
    uint8_t PDPType;
    uint32_t IPAddress;
    uint32_t primaryDNS;
    uint32_t secondaryDNS;
    uint8_t Authentication;
    uint8_t  profileName[MAX_PROFILE_NAME_SIZE];
    uint8_t  APNName[MAX_APN_SIZE];
    uint8_t  userName[MAX_USER_NAME_SIZE];
    uint8_t  password[MAX_FIELD_SIZE];
};

/****************************************************************
*                    GLOBAL DATA
****************************************************************/
#define QMI_GET_SERVICE_FILE_IOCTL  0x8BE0 + 1
#define QMI_MSG_MAX 2048
#define QMI_CMD_TIMEOUT_SEC 20 /* 120 sec */

int wds_fd = -1;
unsigned g_xid = 0;
uint8_t g_modem_index = 0;
uint8_t g_qmi_rsp[QMI_MSG_MAX];
uint8_t g_qmi_req[QMI_MSG_MAX];
uint16_t g_rspLen = 0;
uint8_t g_read_success = FALSE;

uint8_t g_session = 0;
uint32_t g_sessionId = 0;

pthread_mutex_t con_mutex = PTHREAD_MUTEX_INITIALIZER ;
pthread_cond_t con_cond = PTHREAD_COND_INITIALIZER ;

/* device connectivity */
static struct profileIndexesInfo indexInfo;

/****************************************************************
*              FORWARD FUNCTION DECLARATION
****************************************************************/
static void display_device_info( void );
static void display_profile_info( void );
static void display_all_profiles( void );
static void IPUlongToDot( uint32_t IPAddress, char* pIPAddress );


/****************************************************************
*                       FUNCTIONS
****************************************************************/

/****************************************************************
*                    COMMON FUNCTIONS
****************************************************************/
/*
 * Name:     FlushStdinStream
 *
 * Purpose:  Flush the stdin stream
 *
 * Params:   None
 *
 * Return:   None
 *
 * Notes:    fflush does not work for input stream.
 */
void FlushStdinStream( )
{
    int inputChar;

    /* keep on reading until a <New Line> or end of file is received */
    do
    {
        inputChar = getchar();

        #ifdef DBG
        fprintf( stderr,  "inputChar: 0x%x\n", inputChar );
        #endif
    }
    while ( ( inputChar != ENTER_KEY ) &&
            ( inputChar != EOF ) );
}

/*
 * Name:     client_fd
 *
 * Purpose:  open QMI svc file descriptor
 *
 * Params:   svc type
 *
 * Return:   file descriptor
 *
 * Notes:    none.
 */

int client_fd(uint8_t svc)
{
    char szDevicePath[32]={0};
    int fd = -1; 
    snprintf(szDevicePath,31,"/dev/qcqmi%d",g_modem_index);
    fd = open(szDevicePath, O_RDWR);
    if(fd>=0)
    {
        if(ioctl(fd, QMI_GET_SERVICE_FILE_IOCTL, svc)!=0)
        {
            if(fd>=0)
                close(fd);
            return -1;
        }
    }
    else
    {
        fprintf(stderr,"Fail Open:%s\n",szDevicePath);
    }
    return fd;
}

/*
 * Name:     ValidateIPAddressOctet
 *
 * Purpose:  Validates the received  octet of the IP Address.
 *
 * Params:   pIPAddressOctet - Pointer to the buffer containing IP Address
 *                             octet to be validated.
 *           len             - length of the passed buffer
 *
 * Return:   SUCCESS   - In case valid digits are there in the received octet of
 *                       the IP Address
 *           FAIL      - If invalid octet is received
 *
 * Notes:    None
 */
uint8_t ValidateIPAddressOctet( char* pIPAddressOctet, int len )
{
    if( len == 3)
    {
        /* If length of the octet is 3, first digit should be between 0 and 2 */
        if( ( '0' > pIPAddressOctet[0] ) ||
            ( '2' < pIPAddressOctet[0] ) )
        {
            return FAIL;
        }

        /* If first digit is 2 then second digit should not be greater than 5 */
        if( '2' == pIPAddressOctet[0] )
        {
            if( '5' < pIPAddressOctet[1] )
            {
                return FAIL;
            }

            /* If second digit is also 5 then third digit should not be greater
             * than 5.
             */
            if( '5' == pIPAddressOctet[1] )
            {
                if( '5' < pIPAddressOctet[2] )
                {
                    return FAIL;
                }
            }
        }

        if( ( '0' > pIPAddressOctet[1] ) ||
            ( '9' < pIPAddressOctet[1] ) )
        {
            return FAIL;
        }

        if( ( '0' > pIPAddressOctet[2] ) ||
            ( '9' < pIPAddressOctet[2] ) )
        {
            return FAIL;
        }
    }

    if( 2 == len )
    {
        if( ( '0' > pIPAddressOctet[0] ) ||
            ( '9' < pIPAddressOctet[0] ) )
        {
            return FAIL;
        }

        if( ( '0' > pIPAddressOctet[1] ) ||
            ( '9' < pIPAddressOctet[1] ) )
        {
            return FAIL;
        }
    }

    if( 1 == len )
    {
        if( ( '0' > pIPAddressOctet[0] ) ||
            ( '9' < pIPAddressOctet[0] ) )
        {
            return FAIL;
        }
    }
    return SUCCESS;
}

/*
 * Name:     IPUlongToDot
 *
 * Purpose:  Convert the IP address passed by the user in the form of ULONG
 *           value to a DOT format and copy it into the user buffer.
 *
 * Params:   IPAddress  - IP Address to be converted in dot notation.
 *           pIPAddress - Buffer to store IP Address converted to dot notation.
 *
 * Return:   None
 *
 * Notes:    None
 */
static void IPUlongToDot( uint32_t IPAddress, char* pIPAddress )
{
    char  tempBuf[5];
    uint8_t  idx = 0;
    uint8_t  shift = 0;
    uint32_t tempIPAddress = 0;
    int   IPAddr = 0;

    for( idx = 4; idx > 0; idx-- )
    {
        shift = ( idx - 1 ) * 8;
        tempIPAddress = IPAddress >> shift;
        IPAddr = tempIPAddress & IPADDREESS_OCTET_MASK;
        sprintf( tempBuf, "%d", IPAddr );
        strcat( pIPAddress, tempBuf );
        tempIPAddress = 0;
        if( 1 >= idx )
        {
            continue;
        }
        strcat( pIPAddress, "." );
    }
}

/*
 * Name:     IPDotToUlong
 *
 * Purpose:  Convert the IP address passed by the user in dot notation to
 *           a ULONG value.
 *
 * Params:   pIPAddress - IP Address to be converted to ULONG value.
 *           pAddress   - ULONG pointer to store converted IP Address.
 *
 * Return:   SUCCESS   - In case valid IP Address is provided by the user
 *           FAIL      - In case invalid IP Address is provided by the user
 *
 * Notes:    None
 */
uint8_t IPDotToUlong( char* pIPAddress, uint32_t* pAddress )
{
    char *pCharacterOccurence = NULL;
    int   IPAddressOctet = 0;
    uint8_t  len = 0, noOfShift = 8;
    uint32_t IPAddress = 0;

    /* Being here means correct no. of dots are there in the IP  address. Jump
     * to first occurrence of the dot.
     */
    pCharacterOccurence = strtok( pIPAddress,"." );
    while( NULL != pCharacterOccurence )
    {
        len = strlen( pCharacterOccurence );
        if( ( 0 == len ) || ( 3 < len ) )
        {
            #ifdef DBG
            fprintf( stderr, "Incorrect octet length : %d\n",len );
            #endif
            *pAddress = 0;
            return FAIL;
        }

        /* Check if the valid digits have been entered in IP Address */
        if( FAIL == ValidateIPAddressOctet( pCharacterOccurence, len ) )
        {
            #ifdef DBG
            fprintf( stderr, "Incorrect characters in octet : %s\n",
                              pCharacterOccurence );
            #endif
            *pAddress = 0;
            return FAIL;
        }

        IPAddressOctet = atoi( pCharacterOccurence );
        if( 255 < IPAddressOctet )
        {
            #ifdef DBG
            fprintf( stderr, "Incorrect octet value : %d\n",IPAddressOctet );
            #endif
            *pAddress = 0;
            return FAIL;
        }
        len = 0;

        /* Store the octet */
        IPAddress = ( IPAddress << noOfShift ) | IPAddressOctet;

        #ifdef DBG
        fprintf( stderr, "IP Address Octet Value: %s, Integer: %d\n",
                          pCharacterOccurence, IPAddressOctet );
        fprintf( stderr, "IP Address : %lx\n", (unsigned long)IPAddress );
        #endif
        IPAddressOctet = 0;

        /* look for the next dot */
        pCharacterOccurence = strtok( NULL, "." );
    }

    *pAddress = IPAddress;
    #ifdef DBG
    fprintf( stderr, "Final IP Address : %lx\n",(unsigned long)*pAddress );
    #endif
    return SUCCESS;
}

/*
 * Name:     GetIPFromUser
 *
 * Purpose:  Prompt the user to enter the IP address and copy it in the passed
 *           buffer.
 *
 * Return:   SUCCESS   - In case valid IP Address is entered by the user
 *           ENTER_KEY - If enter key is pressed by the user
 *
 * Params:   pAddressString - Name of the address to be retrieved.
 *           pIPAddress     - Buffer to receive the address from user.
 *           pAddress       - Pointer to store received IP address after
 *                            conversion from dot notation to ULONG value.
 *
 * Notes:    None
 */
uint8_t GetIPFromUser( char *pAddressString, char *pIPAddress, uint32_t *pAddress )
{
    int  len = 0;
    uint8_t count = 0, returnCode = 0, IPAddressWrong = FALSE;
    char *pCharacterOccurence = NULL, *pEndOfLine = NULL;

    while(1)
    {
        /* Print the menu */
        fprintf( stderr, "\nPlease enter %s Address in xxx.xxx.xxx.xxx format,"\
                         " or press <Enter> to leave the field as blank:\n%s"\
                         "Address : ", pAddressString, pAddressString );

        /* Clear the buffer */
        memset( pIPAddress, 0, MAX_FIELD_SIZE );

        /* Receive the input from the user */
        fgets( pIPAddress, MAX_FIELD_SIZE, stdin );

        /* If '/n' character is not read, there are more characters in input
         * stream. Clear the input stream.
         */
        pEndOfLine = strchr( pIPAddress, ENTER_KEY );
        if( NULL == pEndOfLine )
        {
            FlushStdinStream();
        }

        #ifdef DBG
        fprintf( stderr, "Entered Address : %s\n", pIPAddress );
        #endif

        /* If only <ENTER> is pressed by the user, return to main menu */
        if( ENTER_KEY == pIPAddress[0] )
        {
            *pAddress = 0;
            return ENTER_KEY;
        }

        len = strlen( pIPAddress );
        pIPAddress[len - 1] = '\0';
        len -= 1;

        /* Validate the address entered by the user */
        /* Check the IP Address Length */
        if( IP_ADDRESS_LEN < len )
        {
            #ifdef DBG
            fprintf( stderr, "Incorrect Address Length : %d\n",len );
            #endif
            continue;
        }

        /* Check if there is nothing followed by a Dot in the IP address or
         * there are two adjacent dots.
         */
        pCharacterOccurence = strchr( pIPAddress,'.' );
        while ( NULL != pCharacterOccurence )
        {
            if( ( '.'  == pCharacterOccurence[1] ) ||
                ( '\0' == pCharacterOccurence[1] ) )
            {
                #ifdef DBG
                fprintf( stderr, "Two Adjacent dots or NULL after a dot:"\
                                  "Wrong IP Address\n" );
                #endif
                IPAddressWrong = TRUE;
                break;
            }
            count++;
            pCharacterOccurence = strchr( ( pCharacterOccurence + 1 ),'.' );
        }

        /* If there are more than three dots in the IP address */
        if( ( count != 3 ) || ( IPAddressWrong == TRUE ) )
        {
            #ifdef DBG
            fprintf( stderr, "Incorrect No. of dots in address : %d\n",count );
            #endif
            IPAddressWrong = FALSE;
            count = 0;
            continue;
        }

        count = 0;

        /* Convert the IP address from DOT notation to ULONG */
        returnCode = IPDotToUlong( pIPAddress, pAddress );

        /* If IP Address is not correct */
        if( SUCCESS != returnCode )
        {
            continue;
        }

        /* Valid IP Address has been retrieved */
        return SUCCESS;
    }
}

/*
 * Name:     GetStringFromUser
 *
 * Purpose:  Prompt the user to enter a string and store it in the received
 *           buffer.
 *
 * Params:   pFieldName - Name of the string to be retrieved.
 *           pBuffer    - Buffer to receive the string from user.
 *           bufLen     - Length of the buffer to receive the string from user.
 *
 * Return:   SUCCESS   - If the user enters a valid value
 *           ENTER_KEY - If the user presses the Enter key
 *
 * Notes:    None
 */
uint8_t GetStringFromUser( char* pFieldName, char* pBuffer, int bufLen )
{
    char *pEndOfLine = NULL;
    int  len = 0;

    while(1)
    {
        /* Print the menu */
        fprintf( stderr, "\nPlease enter %s (up to %d Characters),"\
                         " or press <Enter> to leave the field as blank:\n",
                         pFieldName, ( bufLen - 3 ) );

        /* Clear the buffer */
        memset( pBuffer, 0, bufLen );

        /* Receive the input from the user */
        fgets( pBuffer, bufLen, stdin );

        /* If '/n' character is not read, there are more characters in input
         * stream. Clear the input stream.
         */
        pEndOfLine = strchr( pBuffer, ENTER_KEY );
        if( NULL == pEndOfLine )
        {
            FlushStdinStream();
        }

        #ifdef DBG
        fprintf( stderr, "Entered Value : %s\n", pBuffer );
        #endif

        /* If only <ENTER> is pressed by the user, return to main menu */
        if( ENTER_KEY == pBuffer[0] )
        {
            return ENTER_KEY;
        }

        /* Remove the enter key read at the end of the buffer */
        len = strlen( pBuffer );
        pBuffer[len - 1] = '\0';
        len -= 1;

        /* If the user has entered string of more than 17 characters */
        if( ( bufLen - 3) < len )
        {
            continue;
        }
        return SUCCESS;
    }
}

/*
 * Name:     GetPDPType
 *
 * Purpose:  Prompt the user to enter the PDP Type.
 *
 * Params:   None
 *
 * Return:   PDP Type value provided by the user(between 0 - 3),
 *           or hex value of enter key if pressed by the user.
 *
 * Notes:    None
 */
int GetPDPType()
{
    char PDPType[OPTION_LEN];
    uint8_t PDPTypeValue = 0;

    while(1)
    {
        char *pEndOfLine = NULL;
        int len = 0;
        /* Print the menu */
        fprintf( stderr, "\nPlease enter PDP Type value( 0 - IPv4, 1 - PPP, "\
                         "2 - IPV6, 3 - IPV4V6 ),or press <Enter> to exit:\n"\
                         "Option : ");

        /* Receive the input from the user */
        fgets( PDPType, ( OPTION_LEN ), stdin );

        /* If '/n' character is not read, there are more characters in input
         * stream. Clear the input stream.
         */
        pEndOfLine = strchr( PDPType, ENTER_KEY );
        if( NULL == pEndOfLine )
        {
            FlushStdinStream();
        }

        #ifdef DBG
        fprintf( stderr, "PDP Type Value : %s\n", PDPType );
        #endif

        len = strlen( PDPType );

        /* If only <ENTER> is pressed by the user, return to main menu */
        if( ENTER_KEY == PDPType[0] )
            return ENTER_KEY;

        /* If user has entered an invalid input, prompt again */
        if( len < 1 || 2 < len )
        {
            continue;
        }

        /* Convert the authentication value provided by user into integer */
        PDPType[ len - 1 ] = '\0';
        PDPTypeValue = atoi( PDPType );

        /* Validate the Authentication value */
        if( 3 < PDPTypeValue )
        {
            continue;
        }
        return PDPTypeValue;
    }
}

/*
 * Name:     GetAuthenticationValue
 *
 * Purpose:  Prompt the user to enter the Authentication value.
 *
 * Params:   None
 *
 * Return:   Authentication value provided by the user(between 0 - 3),
 *           or hex value of enter key if pressed by the user.
 *
 * Notes:    None
 */
int GetAuthenticationValue()
{
    char authenticationValue[OPTION_LEN];
    char *pEndOfLine = NULL;
    uint8_t authValue = 0;
    int  len = 0;

    while(1)
    {
        /* Print the menu */
        fprintf( stderr, "\nPlease enter Authentication value( 0 - None, 1 -"\
                         "PAP, 2 - CHAP, 3 - PAP/CHAP ),or press <Enter> to "\
                         "leave the field as blank:\nOption : ");

        /* Receive the input from the user */
        fgets( authenticationValue, ( OPTION_LEN ), stdin );

        /* If '/n' character is not read, there are more characters in input
         * stream. Clear the input stream.
         */
        pEndOfLine = strchr( authenticationValue, ENTER_KEY );
        if( NULL == pEndOfLine )
        {
            FlushStdinStream();
        }

        #ifdef DBG
        fprintf( stderr, "Authentication Value : %s\n", authenticationValue );
        #endif

        len = strlen( authenticationValue );

        /* If only <ENTER> is pressed by the user, return to main menu */
        if( ENTER_KEY == authenticationValue[0] )
            return ENTER_KEY;

        /* If user has entered an invalid input, prompt again */
        if( len < 1 || 2 < len )
        {
            continue;
        }

        /* Convert the authentication value provided by user into integer */
        authenticationValue[ len - 1 ] = '\0';
        authValue = atoi( authenticationValue );

        /* Validate the Authentication value */
        if( 3 < authValue )
        {
            continue;
        }
        return authValue;
    }
}

/*
 * Name:     GetIPFamilyPreference
 *
 * Purpose:  Prompt the user to enter the IP Family preference
 *
 * Params:   None
 *
 * Return:   IPFamilyPreference
 *
 * Notes:    None
 */
int GetIPFamilyPreference()
{
    char selOption[OPTION_LEN];
    char *pEndOfLine = NULL;
    uint8_t IPFamilyPreference = IPv4_FAMILY_PREFERENCE; /* default to IPV4 */
    int len = 0;

    while(1)
    {
        /* Print the menu */
        fprintf( stderr,"\n1. IPV4 (default)\n" \
                        "2. IPV6\n" \
                        "3. IPV4V6\n" \
                        "Please select IP family preference for the call, or press <Enter> to exit: ");

        /* Receive the input from the user */
        fgets( selOption, ( OPTION_LEN ), stdin );

        /* If '/n' character is not read, there are more characters in input
         * stream. Clear the input stream.
         */
        pEndOfLine = strchr( selOption, ENTER_KEY );
        if( NULL == pEndOfLine )
        {
            FlushStdinStream();
        }

        #ifdef DBG
        fprintf( stderr, "Selected Option : %s\n", selOption );
        #endif

        len = strlen( selOption );

        /* If only <ENTER> is pressed by the user, return to main menu */
        if( ENTER_KEY == selOption[0] )
            return ENTER_KEY_PRESSED;

        /* If user has entered an invalid input, prompt again */
        if( 2 > len )
        {
            continue;
        }
        /* Convert the option added by user into integer */
        selOption[ len - 1 ] = '\0';
        int selection = atoi( selOption );

        switch (selection)
        {
            case 1:
                IPFamilyPreference = IPv4_FAMILY_PREFERENCE;
                break;
            case 2:
                IPFamilyPreference = IPv6_FAMILY_PREFERENCE;
                break;
            case 3:
                IPFamilyPreference = IPv4v6_FAMILY_PREFERENCE;
                break;
            default:
                continue; /* reject everything else */
                break;
        }

        if ( (IPFamilyPreference == IPv4_FAMILY_PREFERENCE) ||
             (IPv6_FAMILY_PREFERENCE == IPFamilyPreference) ||
             (IPv4v6_FAMILY_PREFERENCE == IPFamilyPreference) )
        {
            break;
        }
    }
    return IPFamilyPreference;
}

/*
 * Name:     GetUserProfileId
 *
 * Purpose:  Prompt the user to enter the profile id whose information needs to
 *           be retrieved.
 *
 * Params:   None
 *
 * Return:   Profile index selected by the user(between 1 - 16),
 *           or hex value of enter key if pressed by the user else FALSE.
 *
 * Notes:    None
 */
int GetUserProfileId()
{
    char selOption[OPTION_LEN];
    char *pEndOfLine = NULL;
    uint8_t profileIdx = 0;
    int len = 0;

    while(1)
    {
        /* Print the menu */
        fprintf( stderr, "\nPlease provide a profile id(1-16), or press <Enter> to exit: ");

        /* Receive the input from the user */
        fgets( selOption, ( OPTION_LEN ), stdin );

        /* If '/n' character is not read, there are more characters in input
         * stream. Clear the input stream.
         */
        pEndOfLine = strchr( selOption, ENTER_KEY );
        if( NULL == pEndOfLine )
        {
            FlushStdinStream();
        }

        #ifdef DBG
        fprintf( stderr, "Selected Option : %s\n", selOption );
        #endif

        len = strlen( selOption );

        /* If only <ENTER> is pressed by the user, return to main menu */
        if( ENTER_KEY == selOption[0] )
            return ENTER_KEY_PRESSED;

        /* If user has entered an invalid input, prompt again */
        if(len < 1 || 3 < len )
        {
            continue;
        }
        else if( 3 == len )
        {
            /* If the user has provided a 2 digit profile id, validate the
             * second digit as atoi() will ignore it.
             */
            if( ( selOption[ len - 2 ] < '0' ) ||
                ( selOption[ len - 2 ] > '6' ) )
            {
                continue;
            }
        }

        /* Convert the option added by user into integer */
        selOption[ len - 1 ] = '\0';
        profileIdx = atoi( selOption );

        /* If an invalid profile id is provided by the user */
        if( ( MIN_PROFILES > profileIdx ) ||
            ( MAX_PROFILES < profileIdx ) )
        {
            continue;
        }
        else
        {
             break;
        }
    }
    return profileIdx;
}


/******************************************************************************
* Option 8 : Delete a profile stored on the device
******************************************************************************/
static int get_model_id(unpack_dms_GetModelID_t *pModelId)
{
    pack_qmi_t req_ctx;
    unpack_qmi_t rsp_ctx;
    uint16_t rspLen;
    uint16_t reqLen;
    int rtn;

    if (!pModelId) {
        fprintf (stderr, "%s: invalid arg\n", __func__);
        return eLITE_CONNECT_APP_ERR_QMI;
    }    

    memset(&req_ctx, 0, sizeof(req_ctx));
    memset(&rsp_ctx, 0, sizeof(rsp_ctx));
    g_xid++;
    req_ctx.xid = g_xid;
    int dms = client_fd(eDMS);
    if(dms<0)
    {
        fprintf (stderr, "%s: dms fd<0\n", __func__);
        return eLITE_CONNECT_APP_ERR_QMI;
    }
    memset(g_qmi_rsp,0,QMI_MSG_MAX);
    memset(g_qmi_req,0,QMI_MSG_MAX);
    if(pack_dms_GetModelID(&req_ctx, g_qmi_req, &reqLen, NULL)!=0)
    {
        printf("pack_dms_GetModelID error\n");
        close(dms);
        return eLITE_CONNECT_APP_ERR_QMI;
    }
    rtn = write(dms, g_qmi_req, reqLen);
    if (rtn!=reqLen)
    {
        printf("write %d wrote %d\n", reqLen, rtn);
        close(dms);
        return eLITE_CONNECT_APP_ERR_QMI;
    }
    rspLen = read(dms, g_qmi_rsp, QMI_MSG_MAX);
    if(rspLen > 0)
    {
        helper_get_resp_ctx(eDMS, g_qmi_rsp, rspLen, &rsp_ctx);
        if (rsp_ctx.xid == g_xid) {
            rtn = unpack_dms_GetModelID(g_qmi_rsp, rspLen, pModelId);
            if(rtn !=0)
            {
                printf("%s: failed, ret %d\n",__func__,rtn);
                close(dms);
                return eLITE_CONNECT_APP_ERR_QMI;
            }
        }
        else {
           printf("%s: failed,tx id not matched\n",__func__);
           close(dms);
           return eLITE_CONNECT_APP_ERR_QMI;
        }
    }
    else {
       printf("%s: failed,response length error\n",__func__);
       close(dms);
       return eLITE_CONNECT_APP_ERR_QMI;
    }    
    close(dms);
    return eLITE_CONNECT_APP_OK;
}

static int get_home_network(unpack_nas_GetHomeNetwork_t *pHomeNw)
{
    pack_qmi_t req_ctx;
    unpack_qmi_t rsp_ctx;
    uint16_t rspLen;
    uint16_t reqLen;
    int rtn;

    if (!pHomeNw) {
        fprintf (stderr, "%s: invalid arg\n", __func__);
        return eLITE_CONNECT_APP_ERR_QMI;
    }   

    memset(&req_ctx, 0, sizeof(req_ctx));
    memset(&rsp_ctx, 0, sizeof(rsp_ctx));
    g_xid++;
    req_ctx.xid = g_xid;
    int nas = client_fd(eNAS);
    if(nas<0)
    {
        fprintf (stderr, "%s: nas fd<0\n", __func__);
        return eLITE_CONNECT_APP_ERR_QMI;
    }
    memset(g_qmi_rsp,0,QMI_MSG_MAX);
    memset(g_qmi_req,0,QMI_MSG_MAX);
    if(pack_nas_GetHomeNetwork(&req_ctx, g_qmi_req, &reqLen)!=0)
    {
        printf("pack_nas_GetHomeNetwork error\n");
        close(nas);
        return eLITE_CONNECT_APP_ERR_QMI;
    }
    rtn = write(nas, g_qmi_req, reqLen);
    if (rtn!=reqLen)
    {
        printf("write %d wrote %d\n", reqLen, rtn);
        close(nas);
        return eLITE_CONNECT_APP_ERR_QMI;
    }
    rspLen = read(nas, g_qmi_rsp, QMI_MSG_MAX);
    if(rspLen > 0)
    {
        helper_get_resp_ctx(eNAS, g_qmi_rsp, rspLen, &rsp_ctx);
        if (rsp_ctx.xid == g_xid) {
            rtn = unpack_nas_GetHomeNetwork(g_qmi_rsp, rspLen, pHomeNw);
            if(rtn !=0)
            {
                printf("%s: failed, ret %d\n",__func__,rtn);
                close(nas);
                return eLITE_CONNECT_APP_ERR_QMI;
            }
        }
        else {
           printf("%s: failed,tx id not matched\n",__func__);
           close(nas);
           return eLITE_CONNECT_APP_ERR_QMI;
        }
    }
    else {
       printf("%s: failed,response length error\n",__func__);
       close(nas);
       return eLITE_CONNECT_APP_ERR_QMI;
    }    
    close(nas);
    return eLITE_CONNECT_APP_OK;
}

static int delete_profile(
    pack_wds_SLQSDeleteProfile_t *pProfileIn,
    unpack_wds_SLQSDeleteProfile_t *pProfileOut)
{
    pack_qmi_t req_ctx;
    unpack_qmi_t rsp_ctx;
    uint16_t reqLen;
    int rtn;
    struct timespec times;
    uint8_t bResp = 0;
    long long timeout = QMI_CMD_TIMEOUT_SEC;

    if ((!pProfileIn) || (!pProfileOut)) {
        fprintf (stderr, "%s: invalid arg\n", __func__);
        return eLITE_CONNECT_APP_ERR_QMI;
    }   

    memset(&req_ctx, 0, sizeof(req_ctx));
    memset(&rsp_ctx, 0, sizeof(rsp_ctx));
    g_xid++;
    req_ctx.xid = g_xid;
  
    memset(g_qmi_rsp,0,QMI_MSG_MAX);
    memset(g_qmi_req,0,QMI_MSG_MAX);

    clock_gettime(CLOCK_REALTIME, &times);
    times.tv_sec += timeout;

    if(pack_wds_SLQSDeleteProfile(&req_ctx, g_qmi_req, &reqLen, pProfileIn)!=0)
    {
        printf("pack_wds_SLQSDeleteProfile error!\n");
        return eLITE_CONNECT_APP_ERR_QMI;
    }

    pthread_mutex_lock(&con_mutex);    
    g_read_success = FALSE; 
    pthread_mutex_unlock(&con_mutex);

    rtn = write(wds_fd, g_qmi_req, reqLen);
    if (rtn!=reqLen)
    {
        printf("write %d wrote %d\n", reqLen, rtn);
        return eLITE_CONNECT_APP_ERR_QMI;
    }

    /* Wait for timeout or some response for this request */
    pthread_mutex_lock(&con_mutex); 
    pthread_cond_timedwait(&con_cond,&con_mutex, &times);
    if (g_read_success)
        bResp = 1; 
    pthread_mutex_unlock(&con_mutex);
    if(bResp)
    {        
        rtn = unpack_wds_SLQSDeleteProfile(g_qmi_rsp, g_rspLen, pProfileOut);
        if(rtn !=0)
        {
            printf("%s: failed, ret %d\n",__func__,rtn);           
            return eLITE_CONNECT_APP_ERR_QMI;
        }
    }
    else {
       printf("%s: failed,response not received or error\n",__func__);
       return eLITE_CONNECT_APP_ERR_QMI;
    }
    return eLITE_CONNECT_APP_OK;
}

/*
 * Name:     delete_profile_from_device
 *
 * Purpose:  Delete the profile from the device for the profile id provided by
 *           the user.
 *
 * Params:   None
 *
 * Return:   None.
 *
 * Notes:    None
 */
void delete_profile_from_device()
{
    int                            rtn = 0;
    uint8_t                        profileType = PROFILE_TYPE_UMTS;
    uint8_t                        profileId;
    pack_wds_SLQSDeleteProfile_t   deleteProfile;
    unpack_wds_SLQSDeleteProfile_t   deleteProfileOut;

    while(1)
    {
        memset(&deleteProfile,0, sizeof(deleteProfile));
        memset(&deleteProfileOut,0, sizeof(deleteProfileOut));

        deleteProfileOut.extendedErrorCode = (uint16_t)-1;

        /* Display all the profiles stored on the device */
        display_all_profiles();

        /* If no profile exist on the device, return */
        if( 0 == indexInfo.totalProfilesOnDevice )
        {
            fprintf( stderr, "No Profile exist on the device for deletion "\
                             "or check device connectivity\n\n");
            return;
        }
            
        /* Receive the user input */
        profileId = GetUserProfileId();

        /* If only <ENTER> is pressed by the user, return to main menu */
        if( ENTER_KEY_PRESSED == profileId )
            return;

        deleteProfile.profileIndex = profileId;
        deleteProfile.profileType = profileType;

        /* Delete the profile from the device */
        rtn = delete_profile(&deleteProfile, &deleteProfileOut); 

        if (rtn != eLITE_CONNECT_APP_OK)
        {
            fprintf( stderr, "Profile deletion Failed\n"\
                             "Failure cause - %d\n", rtn );
            continue;
        }

        fprintf( stderr, "Profile for index %d deleted successfully\n",
                          profileId );
    }
}

/******************************************************************************
* Option 7 : Modify the settings of an existing profile stored on the device
******************************************************************************/

static int change_profile_setting(
    pack_wds_SLQSModifyProfile_t *pProfileIn,
    unpack_wds_SLQSModifyProfile_t *pProfileOut)
{
    pack_qmi_t req_ctx;
    unpack_qmi_t rsp_ctx;
    uint16_t reqLen;
    int rtn;
    struct timespec times;
    uint8_t bResp = 0;
    long long timeout = QMI_CMD_TIMEOUT_SEC;

    if ((!pProfileIn) || (!pProfileOut)) {
        fprintf (stderr, "%s: invalid arg\n", __func__);
        return eLITE_CONNECT_APP_ERR_QMI;
    }   

    memset(&req_ctx, 0, sizeof(req_ctx));
    memset(&rsp_ctx, 0, sizeof(rsp_ctx));
    g_xid++;
    req_ctx.xid = g_xid;
  
    memset(g_qmi_rsp,0,QMI_MSG_MAX);
    memset(g_qmi_req,0,QMI_MSG_MAX);

    clock_gettime(CLOCK_REALTIME, &times);
    times.tv_sec += timeout;

    if(pack_wds_SLQSModifyProfile(&req_ctx, g_qmi_req, &reqLen, pProfileIn)!=0)
    {
        printf("pack_wds_SLQSModifyProfile error!\n");
        return eLITE_CONNECT_APP_ERR_QMI;
    }

    pthread_mutex_lock(&con_mutex);    
    g_read_success = FALSE; 
    pthread_mutex_unlock(&con_mutex);

    rtn = write(wds_fd, g_qmi_req, reqLen);
    if (rtn!=reqLen)
    {
        printf("write %d wrote %d\n", reqLen, rtn);
        return eLITE_CONNECT_APP_ERR_QMI;
    }

    /* Wait for timeout or some response for this request */
    pthread_mutex_lock(&con_mutex); 
    pthread_cond_timedwait(&con_cond,&con_mutex, &times);
    if (g_read_success)
        bResp = 1; 
    pthread_mutex_unlock(&con_mutex);
    if(bResp)
    {        
        rtn = unpack_wds_SLQSModifyProfile(g_qmi_rsp, g_rspLen, pProfileOut);
        if(rtn !=0)
        {
            printf("%s: failed, ret %d\n",__func__,rtn);           
            return eLITE_CONNECT_APP_ERR_QMI;
        }
    }
    else {
       printf("%s: failed,response not received or error\n",__func__);
       return eLITE_CONNECT_APP_ERR_QMI;
    }
    return eLITE_CONNECT_APP_OK;
}

/*
 * Name:     modify_profile_settings
 *
 * Purpose:  Modify the Profile settings of the profile id selected by the user
 *           with the values entered by the user.
 *
 * Params:   None
 *
 * Return:   None
 *
 * Notes:    None
 */
void modify_profile_settings()
{
    pack_wds_SLQSModifyProfile_t modifyProfile;
    unpack_wds_SLQSModifyProfile_t modifyProfileOut;
    int rtn;
    struct profileInformation profileInfo;
    uint8_t                   profileId = 0, returnCode = 0, idx = 0;
    uint8_t                   profileIdMatch = FALSE;
    char                      IPAddress[MAX_FIELD_SIZE];
    uint16_t                  ProfileSize,APNSize, UserSize, PwdSize;
    uint16_t                  extendedErrorCode;

    profileInfo.profileType = PROFILE_TYPE_UMTS;

    while( 1 )
    {
        memset(&modifyProfile,0, sizeof(modifyProfile));
        memset(&modifyProfileOut,0, sizeof(modifyProfileOut));
        extendedErrorCode = (uint16_t)-1;

        modifyProfileOut.pExtErrorCode = &extendedErrorCode;

        /* Display all the profiles stored on the device */
        display_all_profiles();

        if( 0 == indexInfo.totalProfilesOnDevice )
        {
            fprintf( stderr, "No Profile exist on device for modification"\
                             "or check device connectivity\n\n");
            return;
        }

       while(1)
       {
            /* Prompt the user to enter the profile id whose values need to be
             * modified.
             */
            profileId = GetUserProfileId();

            /* If only <ENTER> is pressed by the user, return to main menu */
            if( ENTER_KEY_PRESSED == profileId )
            {
                return;
            }

            /* If the user has enter an invalid profile id */
            for( idx = 0; idx < indexInfo.totalProfilesOnDevice; idx++ )
            {
                if( profileId == indexInfo.profileIndex[idx] )
                {
                     profileIdMatch = TRUE;
                     break;
                }
            }

            if( profileIdMatch == TRUE )
            {
                break;
            }
        }

        modifyProfile.pProfileId = &profileId;
        modifyProfile.pProfileType = &profileInfo.profileType;
        //modifyProfile.curProfile.SlqsProfile3GPP.pPDPtype = &profileInfo.PDPType;

         /* Get PDP Type */
        profileInfo.PDPType = GetPDPType();
        if( ENTER_KEY != profileInfo.PDPType )
        {
            modifyProfile.curProfile.SlqsProfile3GPP.pPDPtype = &profileInfo.PDPType;
        }

        /* Prompt the user to enter the profile parameter values */
        /* Get the IP Address */
        returnCode = GetIPFromUser( "IP", IPAddress, &profileInfo.IPAddress );
        if( ENTER_KEY != returnCode )
        {
            modifyProfile.curProfile.SlqsProfile3GPP.pIPv4AddrPref = &profileInfo.IPAddress; 
        }

        /* Get the Primary DNS Address */
        returnCode = GetIPFromUser( "PrimaryDNS Address", IPAddress,
                                             &profileInfo.primaryDNS );
        if( ENTER_KEY != returnCode )
        {
            modifyProfile.curProfile.SlqsProfile3GPP.pPriDNSIPv4AddPref = &profileInfo.primaryDNS;
        }
        returnCode = 0;

        /* Get the Secondary DNS Address */
        returnCode = GetIPFromUser( "SecondaryDNS Address", IPAddress,
                                               &profileInfo.secondaryDNS );
        if( ENTER_KEY != returnCode )
        {
            modifyProfile.curProfile.SlqsProfile3GPP.pSecDNSIPv4AddPref = &profileInfo.secondaryDNS;
        }
        returnCode = 0;

        /* Get Authentication From the user */
        profileInfo.Authentication = GetAuthenticationValue();
        if( ENTER_KEY != profileInfo.Authentication )
        {
            modifyProfile.curProfile.SlqsProfile3GPP.pAuthenticationPref = &profileInfo.Authentication;
        }
        returnCode = 0;

        /* Get Profile Name from the user, Max size is 14 characters */
        returnCode = GetStringFromUser( "Profile Name", (char*)profileInfo.profileName,
                                                        MAX_PROFILE_NAME_SIZE );
        if( ENTER_KEY != returnCode )
        {
            modifyProfile.curProfile.SlqsProfile3GPP.pProfilename = profileInfo.profileName;
            ProfileSize = strlen((char*)profileInfo.profileName);
            modifyProfile.curProfile.SlqsProfile3GPP.pProfilenameSize = &ProfileSize;
        }
        returnCode = 0;

        /* Get APN Name from the user */
        returnCode = GetStringFromUser( "APN Name", (char*)profileInfo.APNName,
                                                    MAX_APN_SIZE );
        if( ENTER_KEY != returnCode )
        {
            modifyProfile.curProfile.SlqsProfile3GPP.pAPNName = profileInfo.APNName;
            APNSize = strlen((char*)profileInfo.APNName);
            modifyProfile.curProfile.SlqsProfile3GPP.pAPNnameSize = &APNSize;
        }
        returnCode = 0;

        /* Get User Name from the user */
        returnCode = GetStringFromUser( "User Name", (char*)profileInfo.userName,
                                                     MAX_USER_NAME_SIZE );
        if( ENTER_KEY != returnCode )
        {
            modifyProfile.curProfile.SlqsProfile3GPP.pUsername = profileInfo.userName;
            UserSize = strlen((char*)profileInfo.userName);
            modifyProfile.curProfile.SlqsProfile3GPP.pUsernameSize = &UserSize;
        }
        returnCode = 0;

        /* Get Password from the user */
        returnCode = GetStringFromUser( "Password", (char*)profileInfo.password,
                                                    MAX_FIELD_SIZE );
        if( ENTER_KEY != returnCode )
        {
            modifyProfile.curProfile.SlqsProfile3GPP.pPassword = profileInfo.password;
            PwdSize = strlen((char*)profileInfo.password);
            modifyProfile.curProfile.SlqsProfile3GPP.pPasswordSize = &PwdSize;
        }

        rtn = change_profile_setting(&modifyProfile, &modifyProfileOut); 

        if (rtn != eLITE_CONNECT_APP_OK)
        {
            fprintf( stderr, "Profile Modification Failed\n"\
                             "Failure cause - %d\n", rtn );
            continue;
        }
        fprintf( stderr, "Profile Settings updated successfully for Profile ID:"
                          " %d\n",profileId );
    }
}


/******************************************************************************
* Option 6 : Create a profile on the device
******************************************************************************/
static int create_new_profile(
    pack_wds_SLQSCreateProfile_t *pProfileIn,
    unpack_wds_SLQSCreateProfile_t *pProfileOut)
{
    pack_qmi_t req_ctx;
    unpack_qmi_t rsp_ctx;
    uint16_t reqLen;
    int rtn;
    struct timespec times;
    uint8_t bResp = 0;
    long long timeout = QMI_CMD_TIMEOUT_SEC;

    if ((!pProfileIn) || (!pProfileOut)) {
        fprintf (stderr, "%s: invalid arg\n", __func__);
        return eLITE_CONNECT_APP_ERR_QMI;
    }   

    memset(&req_ctx, 0, sizeof(req_ctx));
    memset(&rsp_ctx, 0, sizeof(rsp_ctx));
    g_xid++;
    req_ctx.xid = g_xid;
  
    memset(g_qmi_rsp,0,QMI_MSG_MAX);
    memset(g_qmi_req,0,QMI_MSG_MAX);

    clock_gettime(CLOCK_REALTIME, &times);
    times.tv_sec += timeout;

    if(pack_wds_SLQSCreateProfile(&req_ctx, g_qmi_req, &reqLen, pProfileIn)!=0)
    {
        printf("pack_wds_SLQSCreateProfile error!\n");
        return eLITE_CONNECT_APP_ERR_QMI;
    }

    pthread_mutex_lock(&con_mutex);    
    g_read_success = FALSE; 
    pthread_mutex_unlock(&con_mutex);

    rtn = write(wds_fd, g_qmi_req, reqLen);
    if (rtn!=reqLen)
    {
        printf("write %d wrote %d\n", reqLen, rtn);
        return eLITE_CONNECT_APP_ERR_QMI;
    }

    /* Wait for timeout or some response for this request */
    pthread_mutex_lock(&con_mutex); 
    pthread_cond_timedwait(&con_cond,&con_mutex, &times);
    if (g_read_success)
        bResp = 1; 
    pthread_mutex_unlock(&con_mutex);
    if(bResp)
    {        
        rtn = unpack_wds_SLQSCreateProfile(g_qmi_rsp, g_rspLen, pProfileOut);
        if(rtn !=0)
        {
            printf("%s: failed, ret %d\n",__func__,rtn);           
            return eLITE_CONNECT_APP_ERR_QMI;
        }
    }
    else {
       printf("%s: failed,response not received or error\n",__func__);
       return eLITE_CONNECT_APP_ERR_QMI;
    }
    return eLITE_CONNECT_APP_OK;
}

/*
 * Name:     create_profile
 *
 * Purpose:  Create the Profile with the values provided by the user.
 *
 * Params:   None
 *
 * Return:   None
 *
 * Notes:    None
 */
void create_profile()
{
    pack_wds_SLQSCreateProfile_t createProfile;
    unpack_wds_SLQSCreateProfile_t createProfileOut;
    PackCreateProfileOut         profileInfoOut;
    wds_profileInfo              curProfile;    
    struct profileInformation    profileInfo;
    int                          rtn;
    uint8_t                      profileId = 0, returnCode = 0;
    uint8_t                      PDPType;
    char                         IPAddress[MAX_FIELD_SIZE];
    uint16_t                     ProfileSize,APNSize, UserSize, PwdSize;

    profileInfo.profileType = PROFILE_TYPE_UMTS;

    while( 1 )
    {
        memset(&createProfile,0, sizeof(createProfile));
        memset(&curProfile,0, sizeof(curProfile));
        memset(&createProfileOut,0, sizeof(createProfileOut));
        memset(&profileInfoOut,0, sizeof(profileInfoOut));

        createProfileOut.pCreateProfileOut = &profileInfoOut;
        createProfileOut.pProfileID = &profileId;

        /* Display all the profiles stored on the device */
        display_all_profiles();

        if( 0 == indexInfo.totalProfilesOnDevice )
        {
            fprintf( stderr, "No Profile exist on device for modification"\
                             "or check device connectivity\n\n");
            return;
        }
        createProfile.pCurProfile = &curProfile;
        createProfile.pProfileType = &profileInfo.profileType;        

        /* Get PDP Type */
        PDPType = GetPDPType();
        if( ENTER_KEY == PDPType )
        {
            return;
        }

        createProfile.pCurProfile->SlqsProfile3GPP.pPDPtype = &PDPType; 

        /* Prompt the user to enter the profile parameter values */
        /* Get the IP Address */
        returnCode = GetIPFromUser( "IP", IPAddress, &profileInfo.IPAddress );
        if( ENTER_KEY != returnCode )
        {
            createProfile.pCurProfile->SlqsProfile3GPP.pIPv4AddrPref = &profileInfo.IPAddress; 
        }

        /* Get the Primary DNS Address */
        returnCode = GetIPFromUser( "PrimaryDNS Address", IPAddress,
                                             &profileInfo.primaryDNS );
        if( ENTER_KEY != returnCode )
        {
            createProfile.pCurProfile->SlqsProfile3GPP.pPriDNSIPv4AddPref = &profileInfo.primaryDNS;
        }
        returnCode = 0;

        /* Get the Secondary DNS Address */
        returnCode = GetIPFromUser( "SecondaryDNS Address", IPAddress,
                                               &profileInfo.secondaryDNS );
        if( ENTER_KEY != returnCode )
        {
            createProfile.pCurProfile->SlqsProfile3GPP.pSecDNSIPv4AddPref = &profileInfo.secondaryDNS;
        }
        returnCode = 0;

        /* Get Authentication From the user */
        profileInfo.Authentication = GetAuthenticationValue();
        if( ENTER_KEY != profileInfo.Authentication )
        {
            createProfile.pCurProfile->SlqsProfile3GPP.pAuthenticationPref = &profileInfo.Authentication;
        }
        returnCode = 0;

        /* Get Profile Name from the user, Max size is 14 characters */
        returnCode = GetStringFromUser( "Profile Name", (char*)profileInfo.profileName,
                                                        MAX_PROFILE_NAME_SIZE );
        if( ENTER_KEY != returnCode )
        {
            createProfile.pCurProfile->SlqsProfile3GPP.pProfilename = profileInfo.profileName;
            ProfileSize = strlen((char*)profileInfo.profileName);
            createProfile.pCurProfile->SlqsProfile3GPP.pProfilenameSize = &ProfileSize;
        }
        returnCode = 0;

        /* Get APN Name from the user */
        returnCode = GetStringFromUser( "APN Name", (char*)profileInfo.APNName,
                                                    MAX_APN_SIZE );
        if( ENTER_KEY != returnCode )
        {
            createProfile.pCurProfile->SlqsProfile3GPP.pAPNName = profileInfo.APNName;
            APNSize = strlen((char*)profileInfo.APNName);
            createProfile.pCurProfile->SlqsProfile3GPP.pAPNnameSize = &APNSize;
        }
        returnCode = 0;

        /* Get User Name from the user */
        returnCode = GetStringFromUser( "User Name", (char*)profileInfo.userName,
                                                     MAX_USER_NAME_SIZE );
        if( ENTER_KEY != returnCode )
        {
            createProfile.pCurProfile->SlqsProfile3GPP.pUsername = profileInfo.userName;
            UserSize = strlen((char*)profileInfo.userName);
            createProfile.pCurProfile->SlqsProfile3GPP.pUsernameSize = &UserSize;
        }
        returnCode = 0;

        /* Get Password from the user */
        returnCode = GetStringFromUser( "Password", (char*)profileInfo.password,
                                                    MAX_FIELD_SIZE );
        if( ENTER_KEY != returnCode )
        {
            createProfile.pCurProfile->SlqsProfile3GPP.pPassword = profileInfo.password;
            PwdSize = strlen((char*)profileInfo.password);
            createProfile.pCurProfile->SlqsProfile3GPP.pPasswordSize = &PwdSize;
        }

        rtn = create_new_profile(&createProfile, &createProfileOut); 

        if (rtn != eLITE_CONNECT_APP_OK)
        {
            fprintf( stderr, "Profile Modification Failed\n"\
                             "Failure cause - %d\n", rtn );
            continue;
        }
        fprintf( stderr, "Profile Settings updated successfully for Profile ID:"
                          " %d\n",profileId );
    }
}

/******************************************************************************
* Option 5 : Display the settings for a particular profile stored on the device
******************************************************************************/

static int get_profile_setting(
    uint8_t profileType,
    uint8_t profileId,
    unpack_wds_SLQSGetProfileSettings_t *pProfile)
{
    pack_qmi_t req_ctx;
    unpack_qmi_t rsp_ctx;
    uint16_t reqLen;
    int rtn;
    struct timespec times;
    uint8_t bResp = 0;
    pack_wds_SLQSGetProfileSettings_t reqarg;
    long long timeout = QMI_CMD_TIMEOUT_SEC;

    if (!pProfile) {
        fprintf (stderr, "%s: invalid arg\n", __func__);
        return eLITE_CONNECT_APP_ERR_QMI;
    }   

    memset(&req_ctx, 0, sizeof(req_ctx));
    memset(&rsp_ctx, 0, sizeof(rsp_ctx));
    g_xid++;
    req_ctx.xid = g_xid;
  
    memset(g_qmi_rsp,0,QMI_MSG_MAX);
    memset(g_qmi_req,0,QMI_MSG_MAX);

    clock_gettime(CLOCK_REALTIME, &times);
    times.tv_sec += timeout;

    reqarg.ProfileId = profileId;
    reqarg.ProfileType = profileType;
    if(pack_wds_SLQSGetProfileSettings(&req_ctx, g_qmi_req, &reqLen, &reqarg)!=0)
    {
        printf("pack_wds_SLQSGetProfileSettings error!\n");
        return eLITE_CONNECT_APP_ERR_QMI;
    }

    pthread_mutex_lock(&con_mutex);    
    g_read_success = FALSE; 
    pthread_mutex_unlock(&con_mutex);

    rtn = write(wds_fd, g_qmi_req, reqLen);
    if (rtn!=reqLen)
    {
        printf("write %d wrote %d\n", reqLen, rtn);
        return eLITE_CONNECT_APP_ERR_QMI;
    }

    /* Wait for timeout or some response for this request */
    pthread_mutex_lock(&con_mutex); 
    pthread_cond_timedwait(&con_cond,&con_mutex, &times);
    if (g_read_success)
        bResp = 1; 
    pthread_mutex_unlock(&con_mutex);
    if(bResp)
    {        
        rtn = unpack_wds_SLQSGetProfileSettings(g_qmi_rsp, g_rspLen, pProfile);
        if(rtn !=0)
        {
            #ifdef DBG
            printf("%s: failed, ret %d\n",__func__,rtn);
            #endif           
            return eLITE_CONNECT_APP_ERR_QMI;
        }
    }
    else {
       printf("%s: failed,response not received or error\n",__func__);
       return eLITE_CONNECT_APP_ERR_QMI;
    }
    return eLITE_CONNECT_APP_OK;
}

/*
 * Name:     display_profile_info
 *
 * Purpose:  Display the profile information for the profile index provided by
 *           the user.
 *
 * Params:   None
 *
 * Return:   None.
 *
 * Notes:    None
 */
static void display_profile_info()
{
    int rtn;
    unpack_wds_SLQSGetProfileSettings_t profileSettingsRsp;
    UnPackGetProfileSettingOut profileSetOut;
    uint8_t profileType = PROFILE_TYPE_UMTS;
    uint8_t  profileId;
    uint8_t PDPType = (uint8_t) -1;
    uint32_t IPAddress = (uint32_t) -1;
    char  bufIPAddress[MAX_FIELD_SIZE];
    uint32_t primaryDNS = (uint32_t) -1;
    char  bufPrimaryDNS[MAX_FIELD_SIZE];
    uint32_t secondaryDNS = (uint32_t) -1;
    char  bufSecondaryDNS[MAX_FIELD_SIZE];
    uint8_t authentication= (uint8_t) -1;
    uint8_t  profileName[MAX_PROFILE_NAME_SIZE];
    uint16_t profileNameSize;
    uint8_t  APNName[MAX_APN_SIZE];
    uint16_t APNNameSize;
    uint8_t  Username[MAX_USER_NAME_SIZE];
    uint16_t UsernameSize;
    uint16_t  extendedErrorCode = (uint16_t)-1;
    int w3 = -3, w5 = -5, w8 = -8, w20 = -20;

    memset( &profileSettingsRsp, 0, sizeof(profileSettingsRsp));
    memset( &profileSetOut, 0, sizeof(UnPackGetProfileSettingOut));
    memset( profileName, 0, MAX_PROFILE_NAME_SIZE );
    memset( APNName, 0, MAX_APN_SIZE );
    memset( Username, 0, MAX_USER_NAME_SIZE );
    profileNameSize = sizeof(profileName);
    APNNameSize = sizeof(APNName);
    UsernameSize = sizeof(Username);
    profileSettingsRsp.pProfileSettings = &profileSetOut;
    profileSettingsRsp.pProfileSettings->curProfile.SlqsProfile3GPP.pPDPtype = 
              &PDPType;
    profileSettingsRsp.pProfileSettings->curProfile.SlqsProfile3GPP.pIPv4AddrPref =
              &IPAddress;
    profileSettingsRsp.pProfileSettings->curProfile.SlqsProfile3GPP.pPriDNSIPv4AddPref = 
              &primaryDNS;
    profileSettingsRsp.pProfileSettings->curProfile.SlqsProfile3GPP.pSecDNSIPv4AddPref = 
              &secondaryDNS;
    profileSettingsRsp.pProfileSettings->curProfile.SlqsProfile3GPP.pAuthenticationPref =
              &authentication;
    profileSettingsRsp.pProfileSettings->curProfile.SlqsProfile3GPP.pProfilename = 
              profileName;
    profileSettingsRsp.pProfileSettings->curProfile.SlqsProfile3GPP.pProfilenameSize = 
              &profileNameSize;
    profileSettingsRsp.pProfileSettings->curProfile.SlqsProfile3GPP.pAPNName = 
              APNName;
    profileSettingsRsp.pProfileSettings->curProfile.SlqsProfile3GPP.pAPNnameSize = 
              &APNNameSize;
    profileSettingsRsp.pProfileSettings->curProfile.SlqsProfile3GPP.pUsername = 
              Username;
    profileSettingsRsp.pProfileSettings->curProfile.SlqsProfile3GPP.pUsernameSize = 
              &UsernameSize;
    profileSettingsRsp.pProfileSettings->pExtErrCode = &extendedErrorCode;

    while(1)
    {
        /* Receive the user input */
        profileId = GetUserProfileId();

        /* If only <ENTER> is pressed by the user, return to main menu */
        if( ENTER_KEY_PRESSED == profileId )
        {
            return;
        }
        rtn = get_profile_setting(profileType, profileId, &profileSettingsRsp);
        if (rtn != eLITE_CONNECT_APP_OK) {
            #ifdef DBG
            fprintf( stderr, "Profile retrieving Failed\n"\
                             "Failure cause - %d\nError Code - %d\n\n",
                             rtn, (int)extendedErrorCode );
            #endif
            extendedErrorCode = (uint16_t)-1;
            continue;
        }
        if (profileSettingsRsp.pProfileSettings->curProfile.SlqsProfile3GPP.pProfilename == NULL)
            strcpy((char*)profileName,"Unknown");
        if (profileSettingsRsp.pProfileSettings->curProfile.SlqsProfile3GPP.pAPNName == NULL)
            strcpy((char*)APNName,"Unknown");
        if (profileSettingsRsp.pProfileSettings->curProfile.SlqsProfile3GPP.pUsername == NULL)
            strcpy((char*)Username,"Unknown");

        /* Reset the buffers */
        memset( bufIPAddress, 0, MAX_FIELD_SIZE );
        memset( bufPrimaryDNS, 0, MAX_FIELD_SIZE );
        memset( bufSecondaryDNS, 0, MAX_FIELD_SIZE );

        /* Convert ULONG to Dot notation for display */
        IPUlongToDot( IPAddress, bufIPAddress );
        IPUlongToDot( primaryDNS, bufPrimaryDNS );
        IPUlongToDot( secondaryDNS, bufSecondaryDNS );

        /* Display the header */
        fprintf( stderr, "%*s%*s%*s%*s%*s%*s%*s%*s%*s\n",
                         w3, "ID", w8, "PDPType", w20, "IPAddress",
                         w20, "PrimaryDNS", w20, "SecondaryDNS", w5, "Auth", w20, "ProfileName",
                         w20, "APNName", w20, "UserName" );

        /* Display the retrieved profile information */
        fprintf( stderr, "%*d%*u%*s%*s%*s%*u%*s%*s%*s\n",
                         w3, profileId, w8, PDPType, w20, bufIPAddress,
                         w20, bufPrimaryDNS, w20, bufSecondaryDNS, w5, authentication,
                         w20, profileName, w20, APNName, w20, Username );

    }
}



/*************************************************************************
* Option 4 : Display all the profiles stored on the device
*************************************************************************/
/*
 * Name:     display_all_profiles
 *
 * Purpose:  Display all the profiles stored on the device.
 *
 * Params:   None
 *
 * Return:   None
 *
 * Notes:    None
 */
static void display_all_profiles()
{
     int rtn;
    unpack_wds_SLQSGetProfileSettings_t profileSettingsRsp;
    UnPackGetProfileSettingOut profileSetOut;
    uint8_t profileType = PROFILE_TYPE_UMTS;
    uint8_t  profileId;
    uint8_t PDPType;
    uint32_t IPAddress;
    char  bufIPAddress[MAX_FIELD_SIZE];
    uint32_t primaryDNS;
    char  bufPrimaryDNS[MAX_FIELD_SIZE];
    uint32_t secondaryDNS;
    char  bufSecondaryDNS[MAX_FIELD_SIZE];
    uint8_t authentication;
    uint8_t  profileName[MAX_PROFILE_NAME_SIZE];
    uint16_t profileNameSize;
    uint8_t  APNName[MAX_APN_SIZE];
    uint16_t APNNameSize;
    uint8_t  Username[MAX_USER_NAME_SIZE];
    uint16_t UsernameSize;
    uint16_t  extendedErrorCode;
    int w3 = -3, w5 = -5, w8 = -8, w20 = -20;

    indexInfo.totalProfilesOnDevice = 0;

    profileNameSize = sizeof(profileName);
    APNNameSize = sizeof(APNName);
    UsernameSize = sizeof(Username);

    /* Display the header */
    fprintf( stderr, "\n%*s%*s%*s%*s%*s%*s%*s%*s%*s\n",
                     w3, "ID", w8, "PDPType", w20, "IPAddress",
                     w20, "PrimaryDNS", w20, "SecondaryDNS", w5, "Auth", w20, "ProfileName",
                     w20, "APNName", w20, "UserName" );

    /* Retrieve the information for all the profiles loaded on the device */
    for( profileId = MIN_PROFILES; profileId <= MAX_PROFILES; profileId++ )
    {
        /* Initialize the buffers */
        memset( &profileSettingsRsp, 0, sizeof(profileSettingsRsp));
        memset( &profileSetOut, 0, sizeof(UnPackGetProfileSettingOut));
        memset( profileName, 0, MAX_PROFILE_NAME_SIZE );
        memset( APNName, 0, MAX_APN_SIZE );
        memset( Username, 0, MAX_USER_NAME_SIZE );
        PDPType = (uint8_t) -1;
        IPAddress = (uint32_t) -1;
        primaryDNS = (uint32_t) -1;
        secondaryDNS = (uint32_t) -1;
        authentication= (uint8_t) -1;
        extendedErrorCode = (uint16_t)-1;

        profileSettingsRsp.pProfileSettings = &profileSetOut;
        profileSettingsRsp.pProfileSettings->curProfile.SlqsProfile3GPP.pPDPtype = 
              &PDPType;
        profileSettingsRsp.pProfileSettings->curProfile.SlqsProfile3GPP.pIPv4AddrPref =
              &IPAddress;
        profileSettingsRsp.pProfileSettings->curProfile.SlqsProfile3GPP.pPriDNSIPv4AddPref = 
              &primaryDNS;
        profileSettingsRsp.pProfileSettings->curProfile.SlqsProfile3GPP.pSecDNSIPv4AddPref = 
              &secondaryDNS;
        profileSettingsRsp.pProfileSettings->curProfile.SlqsProfile3GPP.pAuthenticationPref =
              &authentication;
        profileSettingsRsp.pProfileSettings->curProfile.SlqsProfile3GPP.pProfilename = 
              profileName;
        profileSettingsRsp.pProfileSettings->curProfile.SlqsProfile3GPP.pProfilenameSize = 
              &profileNameSize;
        profileSettingsRsp.pProfileSettings->curProfile.SlqsProfile3GPP.pAPNName = 
              APNName;
        profileSettingsRsp.pProfileSettings->curProfile.SlqsProfile3GPP.pAPNnameSize = 
              &APNNameSize;
        profileSettingsRsp.pProfileSettings->curProfile.SlqsProfile3GPP.pUsername = 
              Username;
        profileSettingsRsp.pProfileSettings->curProfile.SlqsProfile3GPP.pUsernameSize = 
              &UsernameSize;
        profileSettingsRsp.pProfileSettings->pExtErrCode = &extendedErrorCode;

        rtn = get_profile_setting(profileType, profileId, &profileSettingsRsp);
        if (rtn != eLITE_CONNECT_APP_OK) {
            #ifdef DBG
            fprintf( stderr, "Profile retrieving Failed\n"\
                             "Failure cause - %d\nError Code - %d\n\n",
                             rtn, (int)extendedErrorCode );
            #endif
            extendedErrorCode = (uint16_t)-1;
            continue;
        }

        /* Store the profile indexes for successfully retrieved profiles */
        indexInfo.profileIndex[indexInfo.totalProfilesOnDevice] = profileId;
        indexInfo.totalProfilesOnDevice++;

        if (profileSettingsRsp.pProfileSettings->curProfile.SlqsProfile3GPP.pProfilename == NULL)
            strcpy((char*)profileName,"Unknown");
        if (profileSettingsRsp.pProfileSettings->curProfile.SlqsProfile3GPP.pAPNName == NULL)
            strcpy((char*)APNName,"Unknown");
        if (profileSettingsRsp.pProfileSettings->curProfile.SlqsProfile3GPP.pUsername == NULL)
            strcpy((char*)Username,"Unknown");

        /* Reset the buffers */
        memset( bufIPAddress, 0, MAX_FIELD_SIZE );
        memset( bufPrimaryDNS, 0, MAX_FIELD_SIZE );
        memset( bufSecondaryDNS, 0, MAX_FIELD_SIZE );

        /* Convert ULONG to Dot notation for display */
        IPUlongToDot( IPAddress, bufIPAddress );
        IPUlongToDot( primaryDNS, bufPrimaryDNS );
        IPUlongToDot( secondaryDNS, bufSecondaryDNS );

        /* Display the retrieved profile information */
        fprintf( stderr, "%*d%*u%*s%*s%*s%*u%*s%*s%*s\n",
                         w3, profileId, w8, PDPType, w20, bufIPAddress,
                         w20, bufPrimaryDNS, w20, bufSecondaryDNS, w5, authentication,
                         w20, profileName, w20, APNName, w20, Username );
    }
}

/*
 * Name:     display_all_profile
 *
 * Purpose:  Call display_all_profiles to display all the profiles stored on the
 *           device.
 *
 * Params:   None
 *
 * Return:   None
 *
 * Notes:    None
 */
void display_all_profile()
{
    display_all_profiles();
    if( 0 == indexInfo.totalProfilesOnDevice )
    {
        fprintf( stderr, "No Profile exist on the device"\
                         "or check device connectivity\n\n" );
    }
}


/*************************************************************************
* Option 3 : Stop the currently active Data Session
*************************************************************************/

static int stop_data_session(pack_wds_SLQSStopDataSession_t *pSession)
{
    pack_qmi_t req_ctx;
    unpack_qmi_t rsp_ctx;
    uint16_t reqLen;
    int rtn;
    struct timespec times;
    uint8_t bResp = 0;
    long long timeout = QMI_CMD_TIMEOUT_SEC;

    if (!pSession) {
        fprintf (stderr, "%s: invalid arg\n", __func__);
        return eLITE_CONNECT_APP_ERR_QMI;
    }   

    memset(&req_ctx, 0, sizeof(req_ctx));
    memset(&rsp_ctx, 0, sizeof(rsp_ctx));
    g_xid++;
    req_ctx.xid = g_xid;
  
    memset(g_qmi_rsp,0,QMI_MSG_MAX);
    memset(g_qmi_req,0,QMI_MSG_MAX);

    clock_gettime(CLOCK_REALTIME, &times);
    times.tv_sec += timeout;

    if(pack_wds_SLQSStopDataSession(&req_ctx, g_qmi_req, &reqLen, pSession)!=0)
    {
        printf("pack_wds_SLQSStartDataSession error!\n");
        return eLITE_CONNECT_APP_ERR_QMI;
    }

    pthread_mutex_lock(&con_mutex);    
    g_read_success = FALSE; 
    pthread_mutex_unlock(&con_mutex);

    rtn = write(wds_fd, g_qmi_req, reqLen);
    if (rtn!=reqLen)
    {
        printf("write %d wrote %d\n", reqLen, rtn);
        return eLITE_CONNECT_APP_ERR_QMI;
    }

    /* Wait for timeout or some response for this request */
    pthread_mutex_lock(&con_mutex); 
    pthread_cond_timedwait(&con_cond,&con_mutex, &times);
    if (g_read_success)
        bResp = 1; 
    pthread_mutex_unlock(&con_mutex);
    if(bResp)
    {        
        unpack_wds_SLQSStopDataSession_t resp;
        memset(&resp,0,sizeof(unpack_wds_SLQSStopDataSession_t));
        rtn = unpack_wds_SLQSStopDataSession(g_qmi_rsp, g_rspLen,&resp);
        if(rtn !=0)
        {
            printf("%s: failed, ret %d\n",__func__,rtn);           
            return eLITE_CONNECT_APP_ERR_QMI;
        }
    }
    else {
       printf("%s: failed,response not received or error\n",__func__);
       return eLITE_CONNECT_APP_ERR_QMI;
    }
    return eLITE_CONNECT_APP_OK;
}

/*
 * Name:     stop_current_datasession
 *
 * Purpose:  Stop the ongoing data session
 *
 * Params:   None
 *
 * Return:   None
 *
 * Notes:    None
 */
void stop_current_datasession()
{
    pack_wds_SLQSStopDataSession_t stopSession;
    int rtn;

    memset(&stopSession,0, sizeof(stopSession));

    if (g_session) {
        stopSession.psid = &g_sessionId;
        rtn = stop_data_session(&stopSession);

        if (rtn != eLITE_CONNECT_APP_OK) {
            #ifdef DBG
            fprintf( stderr, "data session not stopped\n\n");
            #endif
        }
        else {
            fprintf( stderr, "data session stopped\n\n");
            g_session = FALSE;
        }
    }
    else {
        fprintf( stderr, "no data session active currently!\n\n");
    }
}


/*************************************************************************
 * Option 2 : Start LTE Data Session
 ************************************************************************/

static int start_data_session(
    pack_wds_SLQSStartDataSession_t *pSession,
    unpack_wds_SLQSStartDataSession_t *pSessionOut)
{
    pack_qmi_t req_ctx;
    unpack_qmi_t rsp_ctx;
    uint16_t reqLen;
    int rtn;
    struct timespec times;
    uint8_t bResp = 0;
    long long timeout = QMI_CMD_TIMEOUT_SEC;

    if ((!pSession) || (!pSessionOut)) {
        fprintf (stderr, "%s: invalid arg\n", __func__);
        return eLITE_CONNECT_APP_ERR_QMI;
    }   

    memset(&req_ctx, 0, sizeof(req_ctx));
    memset(&rsp_ctx, 0, sizeof(rsp_ctx));
    g_xid++;
    req_ctx.xid = g_xid;
  
    memset(g_qmi_rsp,0,QMI_MSG_MAX);
    memset(g_qmi_req,0,QMI_MSG_MAX);

    clock_gettime(CLOCK_REALTIME, &times);
    times.tv_sec += timeout;

    if(pack_wds_SLQSStartDataSession(&req_ctx, g_qmi_req, &reqLen, pSession)!=0)
    {
        printf("pack_wds_SLQSStartDataSession error!\n");
        return eLITE_CONNECT_APP_ERR_QMI;
    }

    pthread_mutex_lock(&con_mutex);    
    g_read_success = FALSE; 
    pthread_mutex_unlock(&con_mutex);

    rtn = write(wds_fd, g_qmi_req, reqLen);
    if (rtn!=reqLen)
    {
        printf("write %d wrote %d\n", reqLen, rtn);
        return eLITE_CONNECT_APP_ERR_QMI;
    }

    /* Wait for timeout or some response for this request */
    pthread_mutex_lock(&con_mutex); 
    pthread_cond_timedwait(&con_cond,&con_mutex, &times);
    if (g_read_success)
        bResp = 1; 
    pthread_mutex_unlock(&con_mutex);
    if(bResp)

    {        
        rtn = unpack_wds_SLQSStartDataSession(g_qmi_rsp, g_rspLen, pSessionOut);
        if(rtn !=0)
        {
            printf("%s: failed, ret %d\n",__func__,rtn);           
            return eLITE_CONNECT_APP_ERR_QMI;
        }
    }
    else {
       printf("%s: failed,response not received or error\n",__func__);
       return eLITE_CONNECT_APP_ERR_QMI;
    }
    return eLITE_CONNECT_APP_OK;
}

static int set_IP_family(
    pack_wds_SLQSSetIPFamilyPreference_t *pIPfamily,
    unpack_wds_SLQSSetIPFamilyPreference_t *pIPfamilyOut)
{
    pack_qmi_t req_ctx;
    unpack_qmi_t rsp_ctx;
    uint16_t reqLen;
    int rtn;
    struct timespec times;
    uint8_t bResp = 0;
    long long timeout = QMI_CMD_TIMEOUT_SEC;

    if ((!pIPfamily) || (!pIPfamilyOut)) {
        fprintf (stderr, "%s: invalid arg\n", __func__);
        return eLITE_CONNECT_APP_ERR_QMI;
    }   

    memset(&req_ctx, 0, sizeof(req_ctx));
    memset(&rsp_ctx, 0, sizeof(rsp_ctx));
    g_xid++;
    req_ctx.xid = g_xid;
  
    memset(g_qmi_rsp,0,QMI_MSG_MAX);
    memset(g_qmi_req,0,QMI_MSG_MAX);

    clock_gettime(CLOCK_REALTIME, &times);
    times.tv_sec += timeout;

    if(pack_wds_SLQSSetIPFamilyPreference(&req_ctx, g_qmi_req, &reqLen, pIPfamily)!=0)
    {
        printf("pack_wds_SLQSSetIPFamilyPreference error!\n");
        return eLITE_CONNECT_APP_ERR_QMI;
    }

    pthread_mutex_lock(&con_mutex);    
    g_read_success = FALSE; 
    pthread_mutex_unlock(&con_mutex);

    rtn = write(wds_fd, g_qmi_req, reqLen);
    if (rtn!=reqLen)
    {
        printf("write %d wrote %d\n", reqLen, rtn);
        return eLITE_CONNECT_APP_ERR_QMI;
    }

    /* Wait for timeout or some response for this request */
    pthread_mutex_lock(&con_mutex); 
    pthread_cond_timedwait(&con_cond,&con_mutex, &times);
    if (g_read_success)
        bResp = 1; 
    pthread_mutex_unlock(&con_mutex);
    if(bResp)
    {        
        rtn = unpack_wds_SLQSSetIPFamilyPreference(g_qmi_rsp, g_rspLen, pIPfamilyOut);
        if(rtn !=0)
        {
            printf("%s: failed, ret %d\n",__func__,rtn);           
            return eLITE_CONNECT_APP_ERR_QMI;
        }
    }
    else {
       printf("%s: failed,response not received or error\n",__func__);
       return eLITE_CONNECT_APP_ERR_QMI;
    }
    return eLITE_CONNECT_APP_OK;
}

/*
 * Name:     StartLteCdmaDataSession
 *
 * Purpose:  Starts a LTE or CDMA Data Session
 *
 * Params:   isCdma - TRUE for CDMA connection
 *
 * Return:   None
 *
 * Notes:    None
 */
void start_lte_cdma_datasession(uint8_t isCdma)
{
    pack_wds_SLQSStartDataSession_t startSession;
    unpack_wds_SLQSStartDataSession_t startSessionOut;
    unpack_dms_GetModelID_t modelId;    
    pack_wds_SLQSSetIPFamilyPreference_t setIPfamily;
    unpack_wds_SLQSSetIPFamilyPreference_t setIPfamilyOut;
    uint8_t                       technology = (isCdma) ? TECHNOLOGY_3GPP2: TECHNOLOGY_3GPP;
    uint32_t                      profileId3gpp;
    int                           rtn;
    uint8_t                       IPFamilyPreference = 0, profileIdMatch = FALSE;    
    uint32_t                      failReason = 0;
    uint8_t                       idx = 0;

    if (g_session) {
        fprintf( stderr,"Data session is already established,only one instance supported now\n\n");
        return;
    } 

    /* If connected device is GOBI, return after displaying an error message
     * as LTE data call is not supported on GOBI devices.
     */
    memset(&startSession,0, sizeof(startSession));
    memset(&startSessionOut,0, sizeof(startSessionOut));
    memset(&setIPfamily,0, sizeof(setIPfamily));
    memset(&setIPfamilyOut,0, sizeof(setIPfamilyOut));

    /* If connected device is GOBI, return after displaying an error message
     * as this data call is not supported on GOBI devices.
     */
    get_model_id(&modelId);

    if (strstr(modelId.modelid, "MC83"))
    {
        fprintf( stderr, "LTE/CDMA Data call is not supported on this device!!!\n" );
        return;
    }

    if ( (0 == strcmp("SL9090", modelId.modelid) ||\
          0 == strcmp("MC9090", modelId.modelid)) && (1 == isCdma))
    {
        /* Fill the information for required data session, for SL9090, it only supports mono PDN,
           hence, do not consider mutiple PDN in this case */
        startSession.pTech = &technology;
        startSessionOut.psid = &g_sessionId;
        startSessionOut.pFailureReason = &failReason;
        rtn = start_data_session(&startSession, &startSessionOut);

        if (rtn != eLITE_CONNECT_APP_OK) {
            #ifdef DBG
            fprintf( stderr, "fail to start CDMA data session for SL/MC9090\n"\
                     "Failure cause - %d\nError Code - %u\n\n",
                     rtn, failReason );
            #endif
            return;
        }
        else {
            fprintf( stderr, "CDMA data session started successfully\n");
            g_session = TRUE;
        }
    }
    else
    {
        /* handle data connection for NON-SL9090 modules */
        /* Display all the profiles stored on the device */
        display_all_profiles();

        if( 0 == indexInfo.totalProfilesOnDevice )
        {
            fprintf( stderr, "No Profile exist on the device for Data session\n"
                         "or check device connectivity\n\n");
            return;
        }
        while(1)
        {
            /* Get the profile id using which the data session needs to be started */
            profileId3gpp  = GetUserProfileId();

           /* If only <ENTER> is pressed by the user, return to main menu */
           if( ENTER_KEY_PRESSED == profileId3gpp )
               return;

            /* If the user has enter an invalid profile id */
            for( idx = 0; idx < indexInfo.totalProfilesOnDevice; idx++ )
            {
                if( profileId3gpp == indexInfo.profileIndex[idx] )
                {
                    profileIdMatch = TRUE;
                    break;
                }
            }

            if( profileIdMatch == TRUE )
            {
                break;
            }
        }   

        IPFamilyPreference = GetIPFamilyPreference();
        if( ENTER_KEY_PRESSED == IPFamilyPreference )
            IPFamilyPreference = IPv4_FAMILY_PREFERENCE;

        if (IPFamilyPreference == IPv4v6_FAMILY_PREFERENCE)
            IPFamilyPreference = IPv4_FAMILY_PREFERENCE;

        setIPfamily.IPFamilyPreference = IPFamilyPreference;
        rtn = set_IP_family(&setIPfamily, &setIPfamilyOut);
        if (rtn != eLITE_CONNECT_APP_OK) {
            #ifdef DBG
            fprintf( stderr, "setIPFamily preference failed\n"\
                     "Failure cause - %d\n", rtn);
            #endif
        }

        if (isCdma) {
            profileId3gpp += CDMA_PROFILE_OFFSET;
            startSession.pprofileid3gpp2 = &profileId3gpp;
        }
        else {
            startSession.pprofileid3gpp = &profileId3gpp;
        }
        startSessionOut.psid = &g_sessionId;
        startSessionOut.pFailureReason = &failReason;
        rtn = start_data_session(&startSession, &startSessionOut);

        if (rtn != eLITE_CONNECT_APP_OK) {
            #ifdef DBG
            fprintf( stderr, "failed to start LTE/CDMA data session\n"\
                     "Failure cause - %d\nError Code - %u\n\n",
                     rtn, failReason );
            #endif
            return;
        }
        else {
            fprintf( stderr, "LTE/CDMA data session started successfully for Profile ID:"
                      " %u\n",profileId3gpp );
            g_session = TRUE;
        }
    }
    update_user_display( eCALL_STATUS, "CONNECTED" );
}


/*************************************************************************
 * Option 1 : Start UMTS Data Session
 ************************************************************************/
/*
 * Name:     start_UMTS_data_session
 *
 * Purpose:  Starts a UMTS Data Session
 *
 * Params:   None
 *
 * Return:   None
 *
 * Notes:    None
 */
void start_UMTS_datasession()
{
    pack_wds_SLQSStartDataSession_t startSession;
    unpack_wds_SLQSStartDataSession_t startSessionOut;
    pack_wds_SLQSSetIPFamilyPreference_t setIPfamily;
    unpack_wds_SLQSSetIPFamilyPreference_t setIPfamilyOut;
    uint32_t                       profileId3gpp;
    uint8_t                        profileIdMatch = FALSE;
    uint8_t                        idx = 0;
    uint32_t                       failReason = 0;
    int                            rtn;
    uint8_t                        IPFamilyPreference = IPv4_FAMILY_PREFERENCE;

    if (g_session) {
        fprintf( stderr,"Data session is already established,only one instance supported now\n\n");
        return;
    } 

    /* Display all the profiles stored on the device */
    display_all_profiles();

    if( 0 == indexInfo.totalProfilesOnDevice )
    {
        fprintf( stderr, "No Profile exist on the device for Data session\n"
                         "or check device connectivity\n\n");
        return;
    }

    memset(&startSession,0, sizeof(startSession));
    memset(&startSessionOut,0, sizeof(startSessionOut));
    memset(&setIPfamily,0, sizeof(setIPfamily));
    memset(&setIPfamilyOut,0, sizeof(setIPfamilyOut));

    while(1)
    {
        /* Get the profile id using which the data session needs to be started */
        profileId3gpp  = GetUserProfileId();

        /* If only <ENTER> is pressed by the user, return to main menu */
        if( ENTER_KEY_PRESSED == profileId3gpp )
            return;

        /* If the user has enter an invalid profile id */
        for( idx = 0; idx < indexInfo.totalProfilesOnDevice; idx++ )
        {
            if( profileId3gpp == indexInfo.profileIndex[idx] )
            {
                profileIdMatch = TRUE;
                break;
            }
        }

        if( profileIdMatch == TRUE )
        {
            break;
        }
    }   

    IPFamilyPreference = GetIPFamilyPreference();
    if( ENTER_KEY_PRESSED == IPFamilyPreference )
        IPFamilyPreference = IPv4_FAMILY_PREFERENCE;

    if (IPFamilyPreference == IPv4v6_FAMILY_PREFERENCE)
        IPFamilyPreference = IPv4_FAMILY_PREFERENCE;

    setIPfamily.IPFamilyPreference = IPFamilyPreference;
    rtn = set_IP_family(&setIPfamily, &setIPfamilyOut);
    if (rtn != eLITE_CONNECT_APP_OK) {
        #ifdef DBG
        fprintf( stderr, "setIPFamily preference failed\n"\
                     "Failure cause - %d\n", rtn);
        #endif
    }

    startSession.pprofileid3gpp = &profileId3gpp;
    startSessionOut.psid = &g_sessionId;
    startSessionOut.pFailureReason = &failReason;
    rtn = start_data_session(&startSession, &startSessionOut);

    if (rtn != eLITE_CONNECT_APP_OK) {
        #ifdef DBG
        fprintf( stderr, "UMTS data session Failed\n"\
                     "Failure cause - %d\nError Code - %u\n\n",
                     rtn, failReason );
        #endif
        return;
    }
    else {
        fprintf( stderr, "UMTS data session started successfully for Profile ID:"
                      " %u\n",profileId3gpp );
        g_session = TRUE;
    }

    update_user_display( eCALL_STATUS, "CONNECTED" );
}


/*
 * Name:     start_cdma_datasession_RUIM
 *
 * Purpose:  Starts a CDMA Data Session for RUIM Firmware
 *
 * Params:   None
 *
 * Return:   None
 *
 * Notes:    None
 */
void start_cdma_datasession_RUIM()
{
    pack_wds_SLQSStartDataSession_t startSession;
    unpack_wds_SLQSStartDataSession_t startSessionOut;
    unpack_dms_GetModelID_t modelId;    
    pack_wds_SLQSSetIPFamilyPreference_t setIPfamily;
    unpack_wds_SLQSSetIPFamilyPreference_t setIPfamilyOut;
    uint8_t                        technology = TECHNOLOGY_3GPP2;    
    uint8_t                        IPFamilyPreference = IPv4_FAMILY_PREFERENCE;    
    uint32_t                       failReason = 0;	
    int                            rtn;

    if (g_session) {
        fprintf( stderr,"Data session is already established,only one instance supported now\n\n");
        return;
    } 
    
    memset(&startSession,0, sizeof(startSession));
    memset(&startSessionOut,0, sizeof(startSessionOut));
    memset(&setIPfamily,0, sizeof(setIPfamily));
    memset(&setIPfamilyOut,0, sizeof(setIPfamilyOut));

    /* If connected device is GOBI, return after displaying an error message
     * as this data call is not supported on GOBI devices.
     */
    get_model_id(&modelId);

    if (strstr(modelId.modelid, "MC83"))
    {
        fprintf( stderr, "This CDMA Data call is not supported on this device!!!\n" );
        return;
    }

    setIPfamily.IPFamilyPreference = IPFamilyPreference;
    rtn = set_IP_family(&setIPfamily, &setIPfamilyOut);
    if (rtn != eLITE_CONNECT_APP_OK) {
        #ifdef DBG
        fprintf( stderr, "setIPFamily preference failed\n"\
                     "Failure cause - %d\n", rtn);
        #endif
    }

    startSession.pTech = &technology;
    startSessionOut.psid = &g_sessionId;
    startSessionOut.pFailureReason = &failReason;
    rtn = start_data_session(&startSession, &startSessionOut);

    if (rtn != eLITE_CONNECT_APP_OK) {
        #ifdef DBG
        fprintf( stderr, "Fail to start CDMA/RUIM data session\n"\
                     "Failure cause - %d\nError Code - %u\n\n",
                     rtn, failReason );
        #endif
        return;
    }
    else {
        fprintf( stderr, "CDMA Data Session started successfully\n" );
        g_session = TRUE;
    }

    update_user_display( eCALL_STATUS, "CONNECTED" );
}


/*
 * Name:     display_device_info
 *
 * Purpose:  Display the information about the connected device
 *
 * Params:   None
 *
 * Return:   None
 *
 * Notes:    None
 */
static void display_device_info()
{
    int rtn;
    unpack_dms_GetModelID_t modelId;
    unpack_nas_GetHomeNetwork_t homeNw;
    char  *pNAString = "UNAVAILABLE";
    
    memset( &modelId, 0, sizeof(unpack_dms_GetModelID_t));

    /* Get the Device Model ID */
    rtn = get_model_id(&modelId);
    if( rtn != eLITE_CONNECT_APP_OK)
    {
        #ifdef DBG
        fprintf( stderr, "Failed to get model ID\n" );
        #endif
        update_user_display( eMODEL_ID, pNAString );
    }
    else
    {
        /* Update the model ID field of the user window */
        update_user_display( eMODEL_ID, modelId.modelid );
    }

    memset( &homeNw, 0, sizeof(unpack_nas_GetHomeNetwork));

    /* Get the Home Network */
    rtn = get_home_network(&homeNw);
    if( rtn != eLITE_CONNECT_APP_OK)
    {
        #ifdef DBG
        fprintf( stderr, "Failed to get home network\n" );
        #endif
        update_user_display( eHOME_NETWORK, pNAString );
    }
    else
    {
        /* Update the model ID field of the user window */
        update_user_display( eHOME_NETWORK, homeNw.name );
    }
}

void *wds_read_thread(void* ptr)
{
    unpack_qmi_t rsp_ctx;
    uint8_t rsp[QMI_MSG_MAX];
    uint16_t rspLen = 0;
    int rtn = 0;

    UNUSEDPARAM(ptr);
    while(1)
    {
        if (wds_fd < 0)
            break;
        rtn = read(wds_fd, rsp, QMI_MSG_MAX);
        if (rtn > 0)
        {
            rspLen = (uint16_t)rtn;
            helper_get_resp_ctx(eWDS, rsp, rspLen, &rsp_ctx);
            #ifdef QMI_DBG      
            int j;
            printf("<< receiving QMI WDS \n");
            for (j = 0; j < rspLen; j++)
            {
                printf("%02x ", rsp[j]);
                if ((j+1)%16 == 0)
                {
                    printf("\n");
                }
            }
            printf("\n");
            if (rsp_ctx.type == eIND)
                printf("WDS IND: ");
            else if (rsp_ctx.type == eRSP)
                printf("WDS RSP: ");
            printf("msgid 0x%x, type:%x\n", rsp_ctx.msgid,rsp_ctx.type);
            #endif
            switch(rsp_ctx.msgid)
            {
                case eQMI_WDS_START_NET:
                case eQMI_WDS_STOP_NET:
                case eQMI_WDS_SET_IP_FAMILY:                
                case eQMI_WDS_GET_PROFILE:
                case eQMI_WDS_CREATE_PROFILE:
                case eQMI_WDS_MODIFY_PROFILE:
                case eQMI_WDS_DELETE_PROFILE:
                    /* if this response is request by the waiting main thread */
                    if (rsp_ctx.xid == g_xid) {
                        pthread_mutex_lock(&con_mutex);

                        /* 
                         || g_read_success is used to inform main thread that valid 
                         || response is received and ignore duplicate response as well
                         */
                        if (!g_read_success) {
                            g_read_success = TRUE;
                            memcpy (g_qmi_rsp,rsp, rspLen);
                            g_rspLen = rspLen;
                        }
                        pthread_cond_signal(&con_cond);
                        pthread_mutex_unlock(&con_mutex);
                    }
                    break;
                // TODO: Handle the indications here
                default:
                    break;
            }
        }
        else
        {
           /* break the thread */
           #ifdef DBG
           fprintf(stderr,"\nwds read thread error\n");
           #endif
           break;
        }
    }
    return NULL;
}
/*
 * Name:     main
 *
 * Purpose:  Entry point of the application
 *
 * Params:   None
 *
 * Return:   EXIT_SUCCESS, EXIT_FAILURE on unexpected error
 *
 * Notes:    None
 */
int main( int argc, const char *argv[])
{
    char selOption[OPTION_LEN];
    char *pEndOfLine = NULL;
    unsigned int len = 0;
    uint32_t userOption;
    int ret;  
    pthread_attr_t wds_attr;
    pthread_t wds_tid = 0;

    UNUSEDPARAM(argc);
    UNUSEDPARAM(argv);

    /* Initialize the output log file */
    initialize_display();

    /* Being here means, device is connected, update the required field */
    update_user_display( eDEVICE_STATE, "DEVICE CONNECTED" );

    /* Display the information about the connected device */
    display_device_info();

    /* open wds descriptor */
    wds_fd = client_fd(eWDS);

    if (wds_fd < 0) {
        fprintf( stderr, "%s: WDS svc could not open!\n", __func__ );
        exit(EXIT_FAILURE);
    }

    pthread_attr_init(&wds_attr);

    /* create wds read thread */
    if ((pthread_create(&wds_tid, &wds_attr, wds_read_thread, NULL)) < 0) {
        fprintf( stderr, "%s: WDS read thread not created!\n", __func__ );
        exit(EXIT_FAILURE);
    }

    pthread_attr_destroy(&wds_attr);
    usleep(1000);

    while(1)
    {
        /* Print the menu */
        fprintf( stderr, "\nPlease select one of the following options or press <Enter> to exit:\n"\
                         "1.  Start UMTS Data Session\n"\
                         "2.  Start LTE Data Session\n"\
                         "3.  Start CDMA Data Session\n"\
                         "4.  Start RUIM data session\n"
                         "5.  Stop the currently active Data Session\n"\
                         "6.  Display all the profiles stored on the device\n"\
                         "7.  Display the settings for a particular profile stored on the device\n"\
                         "8.  Create a Profile on the device\n"\
                         "9.  Modify the settings of an existing profile stored on the device\n"\
                         "10. Delete a profile stored on the device\n"\
                         "Option : ");

        /* Receive the input from the user */
        fgets( selOption, ( OPTION_LEN ), stdin );

        /* If '/n' character is not read, there are more characters in input
         * stream. Clear the input stream.
         */
        pEndOfLine = strchr( selOption, ENTER_KEY );
        if( NULL == pEndOfLine )
        {
            FlushStdinStream();
        }

        #ifdef DBG
        fprintf( stderr, "Selected Option : %s\n", selOption );
        #endif

        len = strlen( selOption );

        /* If only <ENTER> is pressed by the user quit Application */
        if( ENTER_KEY == selOption[0] )
        {
            break;
        }
      
        /* check descriptor is still valid or not */
        ret = fcntl(wds_fd, F_GETFD);
        if (ret < 0) {
            fprintf( stderr, "Device seems disconnected, exiting application!\n");
            break;
        }

        /* Convert the option added by user into integer */
        selOption[ len - 1 ] = '\0';
        userOption = atoi( selOption );

        /* Process user input */
        switch( userOption ){
            case eSTART_UMTS_DATA_SESSION:
                start_UMTS_datasession();
                break;

            case eSTART_LTE_DATA_SESSION:
                start_lte_cdma_datasession(0);
                break;

            case eSTART_CDMA_DATA_SESSION:
                start_lte_cdma_datasession(1);
                break;

            case eSTART_CDMA_RUIM_DATASESSION:
                start_cdma_datasession_RUIM();
                break;

            case eSTOP_DATA_SESSION:
                stop_current_datasession();
                break;

            case eDISPLAY_ALL_PROFILES:
                display_all_profile();
                break;

            case eDISPLAY_SINGLE_PROFILE:
                display_profile_info();
                break;

            case eCREATE_PROFILE:
                create_profile();
                break;

            case eMODIFY_PROFILE_SETTINGS:
                modify_profile_settings();
                break;

            case eDELETE_PROFILE:
                delete_profile_from_device();
                break;

            default:
                break;
        }
    }

    close(wds_fd);
    wds_fd = -1;

    /* Wait for wds read thread termination */
    pthread_join(wds_tid, NULL);
    return 0;
}
